from __future__ import absolute_import

import logging


class _Logger(object):
    def __init__(self):
        self.log = logging.getLogger(__name__)

    def get_logger(self, **kwargs):
        return logging.LoggerAdapter(self.log, kwargs)


Logger = _Logger()



from __future__ import absolute_import

from .errors import FeatureUnavailableError, RestitutionTimeoutError, RestitutionResultError
from requests.exceptions import Timeout


def restitution_search(client, query):
    if 'limit' in query.keys():
        if query['limit'] > 100000:
            if 'offset' in query.keys() and (query['limit'] - query['offset']) > 100000:
                query.pop('offset', None)
                query['limit'] = 100000
            else:
                query['limit'] = 100000

    try:
        response = client.post("/api/v3/search/all", query)
        if response.ok:
            return response.json()
        else:
            raise RestitutionResultError(response.text)
    except Timeout as e:
        raise RestitutionTimeoutError("Call to restitution timed out (max: {} seconds).".format(client.DEFAULT_TIMEOUT_SECONDS), {"query": query, "error": e})


def blob_store_fetch(client, bucket, object):
    raise FeatureUnavailableError("Use of the blob store is not yet available from outside mnubo's infrastructure.")


def blob_store_bucket_names(client):
    raise FeatureUnavailableError("Use of the blob store is not yet available from outside mnubo's infrastructure.")



from __future__ import absolute_import


class AiotStudioError(Exception):
    """Base AIoT Studio error."""


class UnauthorizedError(AiotStudioError):
    """Action was not authorized (did you provide correct credentials?)."""


class ConfigurationError(AiotStudioError):
    """Configuration is invalid or not found."""


class FeatureUnavailableError(AiotStudioError):
    """Feature or service is not available in the current context."""


class RestitutionError(AiotStudioError):
    """Base restitution error."""


class RestitutionTimeoutError(RestitutionError):
    """Call to restitution (e.g. from a nested search) timed out."""


class RestitutionResultError(RestitutionError):
    """Call to restitution was successful but result indicated an error."""


class BlobStoreError(AiotStudioError):
    """Base blob storage error."""


class BlobStoreTimeoutError(BlobStoreError):
    """Call to blob storage timed out."""


class BlobStoreBucketNotFound(BlobStoreError):
    """Blob store bucket was not found."""


class BlobStoreObjectNotFound(BlobStoreError):
    """Blob store object was not found."""



from __future__ import absolute_import
import os
import ast
import base64
from requests.auth import HTTPBasicAuth
import requests

try:
    from configparser import ConfigParser
except ImportError:
    from ConfigParser import SafeConfigParser as ConfigParser

from aiotstudio._core.errors import *
from aiotstudio import __version__


CONFIGURATION_SECTION = "DEFAULT"


class AuthenticatedClient(object):
    OAUTH_TOKEN_PATH = "/oauth/token?grant_type=client_credentials&scope=ALL"
    DEFAULT_HEADERS = {
        "Content-Type": "application/json",
        "X-MNUBO-SDK": "python/aiotstudio-" + __version__
    }
    DEFAULT_TIMEOUT_SECONDS = 6 * 60

    def __init__(self, client_id, client_secret, api_url):
        self._client_id = client_id
        self._client_secret = client_secret
        self._session = requests.Session()
        self._api_url = api_url
        self._session.headers.update(self.DEFAULT_HEADERS)

        self._token = self.get_token()

        if self._token is None or "access_token" not in self._token:
            raise UnauthorizedError("AuthenticatedClient requires valid 'client_id' and 'client_secret' credentials for the given API URL.")

        self._session.headers.update({"Authorization": "Bearer " + self._token['access_token']})

    def get_token(self):
        if not self._client_id or not self._client_secret:
            return None

        r = self._session.post(self._api_url + self.OAUTH_TOKEN_PATH, auth=HTTPBasicAuth(self._client_id, self._client_secret))
        r.raise_for_status()
        return r.json()

    def get(self, path, params={}):
        return self._session.get(self._api_url+path, params=params, timeout=self.DEFAULT_TIMEOUT_SECONDS)

    def post(self, path, json_body, params={}):
        return self._session.post(self._api_url + path, json=json_body, params=params, timeout=self.DEFAULT_TIMEOUT_SECONDS)


def get_default_client():
    # type: () -> Client

    if all(k in os.environ for k in ("MNUBO_CLIENT_ID", "MNUBO_CLIENT_SECRET", "MNUBO_API_URL")):
        return AuthenticatedClient(
            os.environ["MNUBO_CLIENT_ID"],
            os.environ["MNUBO_CLIENT_SECRET"],
            os.environ["MNUBO_API_URL"]
        )

    config_file = os.environ.get("MNUBO_CONFIG_FILE", "application.conf")
    global_file = os.path.join(os.path.expanduser("~"), ".settings", "mnubo", "application.conf")
    config = ConfigParser()

    config.read([global_file, config_file])
    if all(config.has_option(CONFIGURATION_SECTION, k) for k in ("mnubo_client_id", "mnubo_client_secret", "mnubo_api_url")):
        return AuthenticatedClient(
            config.get(CONFIGURATION_SECTION, "mnubo_client_id"),
            config.get(CONFIGURATION_SECTION, "mnubo_client_secret"),
            config.get(CONFIGURATION_SECTION, "mnubo_api_url")
        )
    else:
        raise ConfigurationError(
            "Could not find the necessary information to connect."
            "Please enter your client id, client secret and api endpoint in a configuration file (%s or ~/.settings/mnubo/application.conf) or"
            "in MNUBO_CLIENT_ID, MNUBO_CLIENT_SECRET MNUBO_API_URL environment variables." % config_file
        )




import pandas as pd
import numpy as np


def mql_result_to_dataframe(result):
    # type: (dict) -> pd.DataFrame
    _columns = result.get('columns', [])
    _rows = result.get('rows', [])

    columns = [col['label'] for col in _columns]
    dataframe = pd.DataFrame(_rows, columns=columns)
    for column in _columns:
        if column['type'] == 'datetime':
            id_col = column['label']
            dataframe[id_col] = pd.to_datetime(dataframe[id_col], utc=True)
    dataframe = dataframe.fillna(value=np.nan)
    return dataframe



from setuptools import setup

import os
import re
import stat

current_directory = os.path.abspath(os.path.dirname(__file__))

def version():
    with open(os.path.join(current_directory, 'version.meta'), 'r') as v:
        return v.read()

def readme():
    with open(os.path.join(current_directory, 'README.md'), 'r') as v:
        return v.read()

def _readfile(file_name):
    with open(os.path.join(current_directory, file_name), 'r') as v:
        lines = v.readlines()
    return list(filter(lambda x: re.match('^\w+', x), lines))

def requirements():
    return _readfile('requirements.txt')

setup(
    name='aiot-studio',
    version=version(),
    author='mnubo, inc.',
    author_email='support@mnubo.com',
    url='https://smartobjects.mnubo.com/documentation/sdks.html',
    packages=['aiotstudio', 'aiotstudio._core'],
    long_description=readme(),
    long_description_content_type='text/markdown',
    install_requires=requirements(),
    include_package_data=True
)



from __future__ import absolute_import

from aiotstudio._core.types import mql_result_to_dataframe
from aiotstudio._core.logger import Logger
from aiotstudio._core.client import get_default_client
from aiotstudio._core import services

_client = get_default_client()
log = Logger.get_logger(feature="datasource")


def search(query):
    # type: (dict) -> dict
    return services.restitution_search(_client, query)


def search_df(query):
    # type: (dict) -> pandas.DataFrame
    return mql_result_to_dataframe(search(query))[:100000]


def blob_store_fetch(bucket_name, object_name):
    # type: (str, str) -> object
    return services.blob_store_fetch(_client, bucket_name, object_name)


def blob_store_bucket_names():
    # type: () -> List[str]
    return services.blob_store_bucket_names(_client)
